@@ -17,22 +17,22 @@ int main(int argc, char *argv[]) {
 
     // Create a query that matches edible components
     ecs_query_t *q = ecs_query(ecs, {
-        .filter.terms = {{ .id = ecs_pair(ecs_id(Eats), EcsWildcard ) }}
+        .terms = {{ .id = ecs_pair(ecs_id(Eats), EcsWildcard ) }}
     });
 
     // Create a few entities that match the query
-    ecs_entity_t bob = ecs_new_entity(ecs, "Bob");
+    ecs_entity_t bob = ecs_entity(ecs, { .name = "Bob" });
     ecs_set_pair(ecs, bob, Eats, Apples, {10});
     ecs_set_pair(ecs, bob, Eats, Pears, {5});
 
-    ecs_entity_t alice = ecs_new_entity(ecs, "Alice");
+    ecs_entity_t alice = ecs_entity(ecs, { .name = "Alice" });
     ecs_set_pair(ecs, alice, Eats, Apples, {4});
 
     // Iterate the query
     ecs_iter_t it = ecs_query_iter(ecs, q);
     while (ecs_query_next(&it)) {
-        Eats *eats = ecs_field(&it, Eats, 1);
-        ecs_id_t pair = ecs_field_id(&it, 1);
+        Eats *eats = ecs_field(&it, Eats, 0);
+        ecs_id_t pair = ecs_field_id(&it, 0);
         ecs_entity_t food = ecs_pair_second(ecs, pair);
 
         for (int i = 0; i < it.count; i ++) {
