@@ -8,7 +8,7 @@ int main(int, char *[]) {
     // components which means it won't match any entities, but will still be ran
     // once for each call to ecs_progress.
     ecs.system()
-        .iter([](flecs::iter& it) {
+        .run([](flecs::iter& it) {
             std::cout << "delta_time: " << it.delta_time() << std::endl;
         });
 
