@@ -437,7 +437,7 @@ void MonitorAlertInstances(ecs_iter_t *it) {
                         /* If a message was already generated, only regenerate if
                         * query has multiple variables. Variable values could have 
                         * changed, this ensures the message remains up to date. */
-                        generate_message = rit.variable_count > 1;
+                        generate_message = ecs_iter_get_var_count(&rit) > 1;
                     }
                 }
 
@@ -566,14 +566,14 @@ ecs_entity_t ecs_alert_init(
             alert->id = desc->id;
         }
 
-        ecs_component_record_t *cdr = flecs_components_ensure(world, alert->id);
-        ecs_assert(cdr != NULL, ECS_INTERNAL_ERROR, NULL);
-        if (!cdr->type_info) {
+        ecs_component_record_t *cr = flecs_components_ensure(world, alert->id);
+        ecs_assert(cr != NULL, ECS_INTERNAL_ERROR, NULL);
+        if (!cr->type_info) {
             ecs_err("ecs_alert_desc_t::id must be a component");
             goto error;
         }
 
-        ecs_entity_t type = cdr->type_info->component;
+        ecs_entity_t type = cr->type_info->component;
         if (type != ecs_get_parent(world, desc->member)) {
             char *type_name = ecs_get_path(world, type);
             ecs_err("member '%s' is not a member of '%s'", 
@@ -613,7 +613,7 @@ ecs_entity_t ecs_alert_init(
         }
 
         alert->offset = member->offset;
-        alert->size = cdr->type_info->size;
+        alert->size = cr->type_info->size;
         alert->kind = pr->kind;
         alert->ranges = ecs_ref_init(world, desc->member, EcsMemberRanges);
         alert->var_id = var_id;
