@@ -6,11 +6,18 @@
 #ifndef FLECS_ITER_H
 #define FLECS_ITER_H
 
+/* Initialize iterator. */
 void flecs_iter_init(
     const ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_flags8_t fields);
+    bool alloc_resources);
 
+/* Deinitialize iterator. */
+void flecs_iter_free(
+    void *ptr,
+    ecs_size_t size);
+
+/* Allocate zero initialized memory from iterator allocator. */
 void* flecs_iter_calloc(
     ecs_iter_t *it,
     ecs_size_t size,
@@ -22,10 +29,6 @@ void* flecs_iter_calloc(
 #define flecs_iter_calloc_n(it, T, count)\
     flecs_iter_calloc(it, ECS_SIZEOF(T) * count, ECS_ALIGNOF(T))
 
-void flecs_iter_free(
-    void *ptr,
-    ecs_size_t size);
-
 #define flecs_iter_free_t(ptr, T)\
     flecs_iter_free(ptr, ECS_SIZEOF(T))
 
