@@ -1845,11 +1845,12 @@ void flecs_table_delete(
                 
                 /* If neither move nor move_ctor are set, this indicates that 
                  * non-destructive move semantics are not supported for this 
-                 * type. In such cases, we set the move_dtor as ctor_move_dtor, 
-                 * which indicates a destructive move operation. This adjustment 
-                 * ensures compatibility with different language bindings. */
-                if (!ti->hooks.move_ctor && ti->hooks.ctor_move_dtor) {
-                  move_dtor = ti->hooks.ctor_move_dtor;
+                 * type. In such cases when we are performing a non-destructive deletion, 
+                 * we set the move_dtor as ctor_move_dtor, which indicates we are operating
+                 * on memory already cleaned up. This adjustment ensures compatibility
+                 * with different language bindings. */
+                if (!destruct && !ti->hooks.move_ctor) {
+                    move_dtor = ti->hooks.ctor_move_dtor;
                 }
 
                 if (move_dtor) {
@@ -1986,8 +1987,14 @@ void flecs_table_move(
                 flecs_table_invoke_add_hooks(world, dst_table,
                     i_new, &dst_entity, dst_index, 1, construct);
             } else {
+                ecs_type_info_t *ti = dst_column->ti;
+
+                /* If the component doesn't have a move/move_ctor,
+                 * then it doesn't support non-destructive moves and must
+                 * be destructed during its removal.
+                 */
                 flecs_table_invoke_remove_hooks(world, src_table,
-                    src_column, &src_entity, src_index, 1, use_move_dtor);
+                    src_column, &src_entity, src_index, 1, use_move_dtor || !ti->hooks.move_ctor);
             }
         }
 
@@ -2001,8 +2008,11 @@ void flecs_table_move(
     }
 
     for (; (i_old < src_column_count); i_old ++) {
+        ecs_column_t *src_column = &src_columns[i_new];
+        ecs_type_info_t *ti = src_column->ti;
+
         flecs_table_invoke_remove_hooks(world, src_table, &src_columns[i_old], 
-            &src_entity, src_index, 1, use_move_dtor);
+            &src_entity, src_index, 1, use_move_dtor || !ti->hooks.move_ctor);
     }
 
     flecs_table_check_sanity(dst_table);
@@ -2050,10 +2060,55 @@ bool flecs_table_shrink(
 
     bool has_payload = table->data.entities != NULL;
 
-    flecs_table_grow_data(world, table, 0, ecs_table_count(table), NULL);
+    int32_t old_size = table->data.size;
+    int32_t count = table->data.count;
+    if (count == table->data.size) {
+        return has_payload;
+    }
+
+    ecs_allocator_t *a = &world->allocator;
+    ecs_column_t *columns = table->data.columns;
+    ecs_entity_t *entities = table->data.entities;
+
+    if (count) {
+        ecs_assert(table->data.entities != NULL, ECS_INTERNAL_ERROR, NULL);
+        table->data.entities = flecs_alloc_n(a, ecs_entity_t, count);
+        ecs_os_memcpy_n(table->data.entities, entities, ecs_entity_t, count);
+    } else {
+        table->data.entities = NULL;
+    }
+    
+    flecs_free_n(a, ecs_entity_t, old_size, entities);
+
+    int32_t i, column_count = table->column_count;
+    for (i = 0; i < column_count; i ++) {
+        const ecs_type_info_t *ti = columns[i].ti;
+        ecs_size_t component_size = ti->size;
+        ecs_move_t move = ti->hooks.ctor_move_dtor;
+        void *data = columns[i].data;
+
+        if (count) {
+            columns[i].data = flecs_alloc(a, component_size * count);
+
+            if (move) {
+                move(columns[i].data, data, count, ti);
+            } else {
+                ecs_assert(columns[i].data != NULL, ECS_INTERNAL_ERROR, NULL);
+                ecs_os_memcpy(columns[i].data, data, component_size * count);   
+            }
+        } else {
+            columns[i].data = NULL;
+        }
+
+        flecs_free(a, component_size * old_size, data);
+    }
+
+    table->data.size = count;
 
     flecs_increment_table_column_version(world, table);
 
+    flecs_table_mark_table_dirty(world, table, 0);
+
     flecs_table_check_sanity(table);
 
     return has_payload;
@@ -2752,6 +2807,35 @@ bool ecs_table_has_id(
     return ecs_table_get_type_index(world, table, id) != -1;
 }
 
+ecs_entity_t ecs_table_get_target(
+    const ecs_world_t *world,
+    const ecs_table_t *table,
+    ecs_entity_t relationship,
+    int32_t index)
+{
+    flecs_poly_assert(world, ecs_world_t);
+
+    ecs_component_record_t *cr = flecs_components_get(world, 
+        ecs_pair(relationship, EcsWildcard));
+    if (!cr) {
+        return 0;
+    }
+
+    const ecs_table_record_t *tr = flecs_component_get_table(cr, table);
+    if (!tr) {
+        return 0;
+    }
+
+    if (index > tr->count) {
+        return 0;
+    }
+
+    ecs_id_t id = table->type.array[tr->index + index];
+    ecs_assert(ECS_IS_PAIR(id), ECS_INTERNAL_ERROR, NULL);
+    ecs_entity_t tgt = ECS_PAIR_SECOND(id);
+    return flecs_entities_get_alive(world, tgt);
+}
+
 int32_t ecs_table_get_depth(
     const ecs_world_t *world,
     const ecs_table_t *table,
